/*
 * Created by sweetbread on 22.02.2025, 17:56
 * Copyright (c) 2025. All rights reserved.
 * Last modified 22.02.2025, 17:56
 */

package ru.risdeveau.pixeldragon.ui.activity

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.CircularProgressIndicator
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.LinearProgressIndicator
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.TopAppBarDefaults.topAppBarColors
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import coil3.compose.SubcomposeAsyncImage
import coil3.network.NetworkHeaders
import coil3.network.httpHeaders
import coil3.request.ImageRequest
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import ru.risdeveau.pixeldragon.api.getRoom
import ru.risdeveau.pixeldragon.api.getRooms
import ru.risdeveau.pixeldragon.api.mxcToUrl
import ru.risdeveau.pixeldragon.db.Room
import ru.risdeveau.pixeldragon.token
import ru.risdeveau.pixeldragon.ui.theme.PixelDragonTheme
import splitties.init.appCtx

class MainActivity : ComponentActivity() {
    @OptIn(ExperimentalMaterial3Api::class)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        enableEdgeToEdge()
        setContent {
            PixelDragonTheme {
                Scaffold(
                    modifier = Modifier.fillMaxSize(),
                    topBar = {
                        TopAppBar(
                            colors = topAppBarColors(
                                containerColor = MaterialTheme.colorScheme.primaryContainer,
                                titleContentColor = MaterialTheme.colorScheme.primary,
                            ),
                            title = {
                                Text("Top app bar")
                            }
                        )
                    },
                ) { innerPadding ->
                    RoomList(Modifier.padding(innerPadding))
                }
            }
        }
    }
}

@Composable
fun RoomList(modifier: Modifier = Modifier) {
    var list by remember { mutableStateOf(listOf<String>()) }
    val coroutineScope = rememberCoroutineScope()
    val listState = rememberLazyListState()

//    if (itemState.scrollToTop) {
//        LaunchedEffect(coroutineScope) {
//            Log.e("TAG", "TopCoinsScreen: scrollToTop" )
//            listState.scrollToItem(0)
//        }
//    }

    LaunchedEffect(true) {
        coroutineScope.launch {
            withContext(Dispatchers.IO) {
                list = getRooms()
            }
        }
    }

    LazyColumn(modifier = modifier, state = listState) {
        items(list) { rid ->
            RoomItem(rid = rid)
        }

        item {
            if (list.isEmpty()) {
                Text("You have no rooms")
            }
        }
    }
}

@Composable
fun RoomItem(modifier: Modifier = Modifier, rid: String) {
    var room by remember { mutableStateOf<Room?>(null) }
    val scope = rememberCoroutineScope()

    LaunchedEffect(true) {
        scope.launch {
            withContext(Dispatchers.IO) {
                room = getRoom(rid)
            }
        }
    }

    if (room != null) {
        Row(
            modifier
                .padding(8.dp)
                .height(72.dp)
                .fillMaxWidth()
                .background(MaterialTheme.colorScheme.primaryContainer, RoundedCornerShape(16.dp))
                .padding(8.dp)
        ) {
            SubcomposeAsyncImage(
                modifier = Modifier
                    .padding(end = 4.dp)
                    .height(64.dp)
                    .width(64.dp),
                model = ImageRequest.Builder(appCtx)
                    .data(mxcToUrl(room!!.avatarUrl ?: ""))
                    .httpHeaders(
                        NetworkHeaders.Builder()
                            .set("Authorization", "Bearer $token")
                            .set("Cache-Control", "max-age=86400")
                            .build()
                    )
                    .build(),
                contentDescription = room!!.roomId,
                loading = {
                    CircularProgressIndicator()
                }
            )
            Column {
                Text(room!!.name ?: "Unknown", maxLines = 1)
            }
        }
    } else {
        LinearProgressIndicator(Modifier.fillMaxWidth())
    }
}

//@Preview(showBackground = true)
//@Composable
//fun GreetingPreview() {
//    PixelDragonTheme {
//        RoomItem()
//    }
//}

